"""Collection of tests to ensure correct implementation and track progress."""
import time
from fastapi.testclient import TestClient
import pytest

from .test_utils import load_trip_landmarks, log_trip_details
from ..structs.preferences import Preferences, Preference
from ..main import app


@pytest.fixture(scope="module")
def client():
    """Client used to call the app."""
    return TestClient(app)


def test_bellecour(client, request) :   # pylint: disable=redefined-outer-name
    """
    Test n°2 : Custom test in Lyon centre to ensure proper decision making in crowded area.
    
    Args:
        client:
        request:
    """
    start_time = time.time()  # Start timer

    # Step 0: Define the trip preferences
    prefs = Preferences(
        sightseeing = Preference(
            type='sightseeing',
            score=5
        ),
        shopping = Preference(
            type='shopping',
            score=5
        ),
        nature = Preference(
            type='nature',
            score=5
        ),
        max_time_minute=120,
        detour_tolerance_minute=0,
    )
    
    # Define the starting coordinates
    start = [45.7576485, 4.8330241]

    # Step 1: request the list of landmarks in the vicinty of the starting point
    response = client.post(
        "/get/landmarks",
        json={
            "preferences": prefs.model_dump(),
            "start": start
            }
        )
    landmarks = response.json()

    # Step 2: Feed the landmarks to the optimizer to compute the trip
    response = client.post(
        "/optimize/trip",
        json={
            "preferences": prefs.model_dump(),
            "landmarks": landmarks,
            "start": start
            }
        )
    result = response.json()
    landmarks = load_trip_landmarks(client, result['first_landmark_uuid'])

    # Get computation time
    comp_time = time.time() - start_time

    # Add details to report
    log_trip_details(request, landmarks, result['total_time'], prefs.max_time_minute)

    # checks :
    assert response.status_code == 200  # check for successful planning
    assert comp_time < 30, f"Computation time exceeded 30 seconds: {comp_time:.2f} seconds"
    assert prefs.max_time_minute*0.8 < result['total_time'], f"Trip too short: {result['total_time']} instead of {prefs.max_time_minute}"
    assert prefs.max_time_minute*1.2 > result['total_time'], f"Trip too long: {result['total_time']} instead of {prefs.max_time_minute}"
